#!/usr/bin/env python3
# www.jrodal.com

import argparse
import asyncio
import logging

from tldrwl.summarize import Summarizer


class VerboseFilter(logging.Filter):
    def __init__(self, name: str = "", very_verbose_logging: bool = False) -> None:
        super().__init__(name)
        self.very_verbose_logging = very_verbose_logging

    def filter(self, record: logging.LogRecord):
        return (
            record.name.startswith("tldrwl")
            or record.levelno != logging.DEBUG
            or self.very_verbose_logging
        )


def _init_logging(args: argparse.Namespace) -> None:
    if args.disable_logging:
        return

    log_level = logging.INFO
    if args.verbose_logging or args.very_verbose_logging:
        log_level = logging.DEBUG

    logging.basicConfig(level=log_level)  # or any other desired logging configuration

    for handler in logging.root.handlers:
        handler.addFilter(VerboseFilter(very_verbose_logging=args.very_verbose_logging))


async def main():
    parser = argparse.ArgumentParser(description="Input (Text or URL to summarize)")
    parser.add_argument("input", type=str, help="Input (Text or URL to summarize)")
    parser.add_argument(
        "--disable-logging", action="store_true", help="Turn logging off"
    )
    parser.add_argument(
        "--verbose-logging", action="store_true", help="verbose logging (tldrwl debug)"
    )
    parser.add_argument(
        "--very-verbose-logging",
        action="store_true",
        help="very verbose logging (include third party logs)",
    )
    args = parser.parse_args()

    _init_logging(args)

    summary = await Summarizer().summarize_async(args.input)
    print(f"Summary: {summary.text}")
    print(f"Estimated cost (usd): ${summary.estimated_cost_usd:.4f}")


if __name__ == "__main__":
    asyncio.run(main())



#!/usr/bin/env python3
# www.jrodal.com

import logging
import re
from typing import Optional, Pattern

from youtube_transcript_api import YouTubeTranscriptApi  # pyright: ignore
from youtube_transcript_api.formatters import TextFormatter  # pyright: ignore

from .ai_interface import AiInterface, Summary
from .exception import TldrwlVideoUrlParsingException
from .summarize_text import TextSummarizer


class YoutubeSummarizer(AiInterface):
    _pattern: Pattern[str] = re.compile(
        r"(?:[?&]v=|\/embed\/|\/1\/|\/v\/|https:\/\/(?:www\.)?youtu\.be\/)([^&\n?#]+)"
    )

    def __init__(self) -> None:
        super().__init__()
        self._text_summarizer = TextSummarizer()
        self._logger = logging.getLogger(__name__)

    @classmethod
    def get_video_id(cls, url: str) -> Optional[str]:
        match = cls._pattern.search(url)
        if match:
            return match.group(1)
        return None

    def _get_video_transcript(self, url: str) -> str:
        video_id = self.get_video_id(url)
        if not video_id:
            raise TldrwlVideoUrlParsingException.make_error(video_url=url)
        self._logger.debug(f"Getting transcript for {video_id}")
        transcript = YouTubeTranscriptApi.get_transcript(video_id)  # type: ignore
        self._logger.debug(f"Done getting transcript for {video_id}")
        return TextFormatter().format_transcript(transcript)  # type: ignore

    async def _summarize_async(self, text: str) -> Summary:
        transcript = self._get_video_transcript(text)
        return await self._text_summarizer.summarize_async(transcript)

    def _summarize_sync(self, text: str) -> Summary:
        transcript = self._get_video_transcript(text)
        return self._text_summarizer.summarize_sync(transcript)



#!/usr/bin/env python3
# www.jrodal.com

import asyncio
import time
import logging
import re
import textwrap

import openai
from typing import List, Dict, Any

from .ai_interface import AiInterface, Model, Summary

MAX_TOKEN_RESPONSE = 1500


class TextSummarizer(AiInterface):
    def __init__(
        self,
        *,
        model: Model = Model.GPT35TURBO,
        prompt_string: str = "Write a detailed summary of the following:\n\n{}\n",
        chunk_size: int = 12000,
        max_num_chunks: int = 10,
    ) -> None:
        super().__init__()
        self._model = model
        self._prompt_string = prompt_string
        self._chunk_size = chunk_size
        self._max_num_chunks = max_num_chunks
        self._logger = logging.getLogger(__name__)

    def _response_to_text_summary(self, response: Dict[str, Any]) -> Summary:
        output_text = response["choices"][0]["message"]["content"]  # type: ignore
        num_tokens = response["usage"]["total_tokens"]  # type: ignore
        self._logger.debug(f"{num_tokens=}")

        summary = re.sub(r"\s+", " ", output_text.strip())  # type: ignore
        return Summary(
            text=summary,
            num_tokens=num_tokens,  # type: ignore
            model=self._model,
        )

    async def _summarize_chunk_async(self, chunk: str, max_tokens: int) -> Summary:
        prompt = self._prompt_string.format(chunk)

        for _ in range(0, 3):
            try:
                response = await openai.ChatCompletion.acreate(  # type: ignore
                    model=self._model.value,
                    messages=[{"role": "user", "content": prompt}],
                    max_tokens=max_tokens,
                )
                return self._response_to_text_summary(response)  # type: ignore
            except openai.error.RateLimitError:  # pyright: ignore
                retry_interval = 3
                self._logger.debug(
                    f"Rate limited by openai - resting for {retry_interval}s"
                )
                await asyncio.sleep(retry_interval)

        return Summary(text="", num_tokens=0, model=self._model)

    def _summarize_chunk(self, chunk: str, max_tokens: int) -> Summary:
        prompt = self._prompt_string.format(chunk)

        for _ in range(0, 3):
            try:
                response = openai.ChatCompletion.create(  # type: ignore
                    model=self._model.value,
                    messages=[{"role": "user", "content": prompt}],
                    max_tokens=max_tokens,
                )
                return self._response_to_text_summary(response)  # type: ignore
            except openai.error.RateLimitError:  # pyright: ignore
                retry_interval = 3
                self._logger.debug(
                    f"Rate limited by openai - resting for {retry_interval}s"
                )
                time.sleep(retry_interval)

        return Summary(text="", num_tokens=0, model=self._model)

    def _get_chunks(self, text: str) -> List[str]:
        text_length = len(text)
        self._logger.debug(f"{text_length=}")

        chunks = textwrap.wrap(text, self._chunk_size)[: self._max_num_chunks]
        num_chunks = len(chunks)
        self._logger.debug(f"{num_chunks=}")

        return chunks

    async def _summarize_async(self, text: str) -> Summary:
        chunks = self._get_chunks(text)
        summaries = await asyncio.gather(
            *[self._summarize_chunk_async(chunk, max_tokens=250) for chunk in chunks]
        )
        if len(summaries) == 0:
            return Summary(text="", num_tokens=0, model=self._model)
        elif len(summaries) == 1:
            return summaries[0]
        else:
            final_input = " ".join(s.text for s in summaries)
            # TODO: recursively summarize if it is still too big instead
            # of hoping that this will work - maybe catch this exception?
            # noqa openai.error.InvalidRequestError: This model's maximum context length is 4097 tokens. However, you requested 4612 tokens (3112 in the messages, 1500 in the completion). Please reduce the length of the messages or completion.
            final_summary = await self._summarize_chunk_async(
                final_input, max_tokens=MAX_TOKEN_RESPONSE
            )
            return Summary(
                text=final_summary.text,
                num_tokens=final_summary.num_tokens
                + sum(s.num_tokens for s in summaries),
                model=self._model,
            )

    def _summarize_sync(self, text: str) -> Summary:
        chunks = self._get_chunks(text)

        summaries = [self._summarize_chunk(chunk, max_tokens=250) for chunk in chunks]
        if len(summaries) == 0:
            return Summary(text="", num_tokens=0, model=self._model)
        elif len(summaries) == 1:
            return summaries[0]
        else:
            final_input = " ".join(s.text for s in summaries)
            final_summary = self._summarize_chunk(
                final_input, max_tokens=MAX_TOKEN_RESPONSE
            )
            return Summary(
                text=final_summary.text,
                num_tokens=final_summary.num_tokens
                + sum(s.num_tokens for s in summaries),
                model=self._model,
            )



#!/usr/bin/env python3
# www.jrodal.com

import logging

from .ai_interface import AiInterface, Summary
from .summarize_text import TextSummarizer
from .summarize_webpage import WebpageSummarizer
from .summarize_youtube import YoutubeSummarizer


class Summarizer(AiInterface):
    def __init__(self) -> None:
        super().__init__()
        self._logger = logging.getLogger(__name__)

    def get_summarizer(self, text: str) -> AiInterface:
        if YoutubeSummarizer.get_video_id(text):
            self._logger.debug("Using YoutubeSummarizer")
            return YoutubeSummarizer()
        elif WebpageSummarizer.is_url(text):
            self._logger.debug("Using WebpageSummarizer")
            return WebpageSummarizer()
        else:
            self._logger.debug("Using TextSummarizer")
            return TextSummarizer()

    async def _summarize_async(self, text: str) -> Summary:
        summarizer = self.get_summarizer(text)
        return await summarizer.summarize_async(text)

    def _summarize_sync(self, text: str) -> Summary:
        summarizer = self.get_summarizer(text)
        return summarizer.summarize_sync(text)



#!/usr/bin/env python3
# www.jrodal.com

from abc import ABC, abstractmethod
from dataclasses import dataclass

from .exception import TldrwlException
from .register import Register
from enum import Enum


class Model(Enum):
    GPT35TURBO = "gpt-3.5-turbo"

    @property
    def cost_per_1000_tokens(self) -> float:
        if self is self.GPT35TURBO:
            return 0.002
        else:
            return 0


@dataclass
class Summary:
    text: str
    num_tokens: int
    model: Model

    @property
    def estimated_cost_usd(self) -> float:
        return self.num_tokens * self.model.cost_per_1000_tokens * (1 / 1000)


class AiInterface(ABC):
    def __init_subclass__(cls) -> None:
        if not Register.is_registered():
            Register.register()

    @abstractmethod
    def _summarize_sync(self, text: str) -> Summary:
        pass

    @abstractmethod
    async def _summarize_async(self, text: str) -> Summary:
        pass

    def summarize_sync(self, text: str) -> Summary:
        try:
            return self._summarize_sync(text)
        except TldrwlException:
            raise
        except Exception as e:
            raise TldrwlException(msg=str(e), cause="n/a", remediation="n/a") from e

    async def summarize_async(self, text: str) -> Summary:
        try:
            return await self._summarize_async(text)
        except TldrwlException:
            raise
        except Exception as e:
            raise TldrwlException(msg=str(e), cause="n/a", remediation="n/a") from e



#!/usr/bin/env python3
# www.jrodal.com

import re
import logging
import bs4
import requests
import aiohttp
from typing import Pattern

from .ai_interface import AiInterface, Summary
from .summarize_text import TextSummarizer


class WebpageSummarizer(AiInterface):
    _pattern: Pattern[str] = re.compile(
        r"^(?:http|ftp)s?://"  # http:// or https://
        r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|"  # noqa domain
        r"localhost|"  # localhost...
        r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"  # ...or ip
        r"(?::\d+)?"  # optional port
        r"(?:/?|[/?]\S+)$",
        re.IGNORECASE,
    )

    def __init__(self) -> None:
        super().__init__()
        self._text_summarizer = TextSummarizer()
        self._logger = logging.getLogger(__name__)

    @classmethod
    def is_url(cls, text: str) -> bool:
        return cls._pattern.match(text) is not None

    def _get_page_text(self, url: str) -> str:
        self._logger.debug(f"Getting page text for {url}")
        response = requests.get(url)
        response.raise_for_status()
        soup = bs4.BeautifulSoup(response.text, "html.parser")
        page_text = soup.get_text()
        self._logger.debug(f"Done getting page text for {url}")
        return page_text

    async def _get_page_text_async(self, url: str) -> str:
        self._logger.debug(f"Getting page text for {url}")
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                response.raise_for_status()
                soup = bs4.BeautifulSoup(await response.text(), "html.parser")
                page_text = soup.get_text()
                self._logger.debug(f"Done getting page text for {url}")
                return page_text

    async def _summarize_async(self, text: str) -> Summary:
        page_text = await self._get_page_text_async(text)
        return await self._text_summarizer.summarize_async(page_text)

    def _summarize_sync(self, text: str) -> Summary:
        page_text = self._get_page_text(text)
        return self._text_summarizer.summarize_sync(page_text)
